﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Encodings.SchedulingEncoding.Encodings;
using HeuristicLab.Encodings.SchedulingEncoding.Interfaces;

namespace HeuristicLab.Encodings.SchedulingEncoding.JobSequenceMatrix.Manipulators {
  [Item("AlbaManipulator", "An operator which manipulates a VRP representation.")]
  [StorableClass]
  public abstract class JSMManipulator : SchedulingManipulator<JSMEncoding>, IJSMOperator {
    [StorableConstructor]
    protected JSMManipulator(bool deserializing) : base(deserializing) { }
    protected JSMManipulator(JSMManipulator original, Cloner cloner) : base(original, cloner) { }
    public JSMManipulator()
      : base() {
    }

    protected abstract JSMEncoding Manipulate(IRandom random, JSMEncoding individual);


    public override IOperation Apply() {
      JSMEncoding solution = SchedulingSolutionParameter.ActualValue;

      int nrOfMutations = RandomParameter.ActualValue.Next(5) + 1;

      for (int i = 0; i < nrOfMutations; i++ )
        SchedulingSolutionParameter.ActualValue = Manipulate(RandomParameter.ActualValue, solution as JSMEncoding);

      return base.Apply();
    }

  }
}
