﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Encodings.PermutationEncoding;

namespace HeuristicLab.Encodings.SchedulingEncoding.JobSequenceMatrix.Manipulators {

  [Item("JSM Swap Manipulator", "Represents a manipulation operation inserting parts of the individual at another position.")]
  [StorableClass]
  public class JSMSwapManipulator : JSMManipulator {
    [StorableConstructor]
    protected JSMSwapManipulator(bool deserializing) : base(deserializing) { }
    protected JSMSwapManipulator(JSMSwapManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new JSMSwapManipulator(this, cloner);
    }

    public JSMSwapManipulator () : base () {}

    protected override JSMEncoding Manipulate(IRandom random, JSMEncoding individual) {
      JSMEncoding result = (JSMEncoding)individual.Clone();
      int resourceIndex = random.Next(result.JobSequenceMatrix.Count);
      Permutation p = result.JobSequenceMatrix[resourceIndex];
      int seqIndex1 = random.Next(p.Length);
      int seqIndex2 = random.Next(p.Length);
      int aux = p[seqIndex1];
      p[seqIndex1] = p[seqIndex2];
      p[seqIndex2] = aux;
      return result;
    }
  }
}
