﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Data;
using HeuristicLab.Encodings.SchedulingEncoding.Interfaces;
using HeuristicLab.Encodings.IntegerVectorEncoding;

namespace HeuristicLab.Encodings.SchedulingEncoding.PriorityRulesVector {
  [Item("Priority Rules Vector Encoding", "Represents a solution for a standard JobShop Scheduling Problem.")]
  [StorableClass]
  public class PRVEncoding : ParameterizedNamedItem, ISchedulingEncoding {
    [StorableConstructor]
    protected PRVEncoding(bool deserializing) : base(deserializing) { }
    protected PRVEncoding(PRVEncoding original, Cloner cloner)
      : base(original, cloner) {
        this.nrOfRules = cloner.Clone(original.NrOfRules);
        this.PriorityRulesVector = cloner.Clone(original.PriorityRulesVector);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PRVEncoding(this, cloner);
    }

    [Storable]
    public IntegerVector PriorityRulesVector { get; set; }

    [Storable]
    private IntValue nrOfRules;
    public IntValue NrOfRules { 
      get {
        return nrOfRules;
      } 
    }


    public PRVEncoding(IntValue nrOfRules)
      : base() {
        this.nrOfRules = (IntValue)nrOfRules.Clone();
        this.PriorityRulesVector = new IntegerVector();
    }

    public PRVEncoding (IntegerVector iv, IntValue nrOfRules) : base () {
      this.nrOfRules = (IntValue)nrOfRules.Clone();
      this.PriorityRulesVector = (IntegerVector)iv.Clone();
    }

    public PRVEncoding(int length, IRandom random, int min, int max, IntValue nrOfRules) : base() {
      this.nrOfRules = (IntValue)nrOfRules.Clone();
      this.PriorityRulesVector = new IntegerVector(length, random, min, max);
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[ ");

      foreach (int i in PriorityRulesVector) {
        sb.Append(i.ToString() + " ");
      }

      sb.Append("]");
      return sb.ToString();
    }
  }
}
