﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Encodings.IntegerVectorEncoding;


namespace HeuristicLab.Encodings.SchedulingEncoding.PriorityRulesVector {
  [Item("Priority Rules Random Creator", "Creator class used to create PRV solutions for standard JobShop scheduling problems.")]
  [StorableClass]
  public class PRVRandomCreator : SchedulingCreator<PRVEncoding>, IStochasticOperator {
    [StorableConstructor]
    protected PRVRandomCreator(bool deserializing) : base(deserializing) { }
    protected PRVRandomCreator(PRVRandomCreator original, Cloner cloner)
      : base(original, cloner) {
        this.NrOfRules = cloner.Clone(original.NrOfRules);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PRVRandomCreator(this, cloner);
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public IValueLookupParameter<IntValue> JobsParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Jobs"]; }
    }
    public IValueLookupParameter<IntValue> ResourcesParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Resources"]; }
    }


    [Storable]
    public IntValue NrOfRules { get; set; }

    public PRVRandomCreator()
      : base() {
      NrOfRules = new IntValue(10);
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("Jobs", "The number of jobs handled in this problem instance."));
      Parameters.Add(new ValueLookupParameter<IntValue>("Resources", "The number of resources used in this problem instance."));
    }


    protected override PRVEncoding CreateSolution() {
      IntValue nrOfJobs = new IntValue (JobsParameter.ActualValue.Value);
      IntValue nrOfResources = new IntValue (ResourcesParameter.ActualValue.Value);
      PRVEncoding solution = new PRVEncoding(nrOfJobs.Value * nrOfResources.Value, RandomParameter.ActualValue, 0, NrOfRules.Value, NrOfRules);
      return solution;
    }
  }
}
