﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.Encodings.SchedulingEncoding {
  [Item("Indexed Tasklist", "Represents a list of tasks.")]
  [StorableClass]
  public class IndexedScheduledTaskList : ParameterizedNamedItem {
    [StorableConstructor]
    protected IndexedScheduledTaskList(bool deserializing) : base(deserializing) { }
    protected IndexedScheduledTaskList(IndexedScheduledTaskList original, Cloner cloner)
      : base(original, cloner) {
      this.Index = cloner.Clone(original.Index);
      this.Tasks = cloner.Clone(original.Tasks);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new IndexedScheduledTaskList(this, cloner);
    }


    public IntValue Index {
      get;
      set;
    }
    public ItemList<ScheduledTask> Tasks {
      get;
      set;
    }


    public DoubleValue TotalDuration {
      get {
        double result = 0;
        foreach (ScheduledTask t in Tasks) {
          if (t.EndTime.Value > result)
            result = t.EndTime.Value;
        }
        return new DoubleValue(result);
      }
    }

    public IndexedScheduledTaskList (IntValue index) : base (){
      Index = index;
      Tasks = new ItemList<ScheduledTask>();
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("TaskList#" + Index + " [ ");
      foreach (ScheduledTask t in Tasks) {
        sb.Append(t.ToString() + " ");
      }
      sb.Append("]");
      return sb.ToString();
    }
  }
}
