﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Encodings.SchedulingEncoding {
  [Item("Scheduled Task Class", "Represents a task that has been scheduled already.")]
  [StorableClass]
  public class ScheduledTask : Item {
    [StorableConstructor]
    protected ScheduledTask(bool deserializing) : base(deserializing) { }
    protected ScheduledTask(ScheduledTask original, Cloner cloner)
      : base(original, cloner) {
      this.ResourceNr = cloner.Clone(original.ResourceNr);
      this.Duration = cloner.Clone(original.Duration);
      this.StartTime = cloner.Clone(original.StartTime);
      this.JobNr = cloner.Clone (original.JobNr);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ScheduledTask(this, cloner);
    }


    public IntValue ResourceNr { get; set; }
    public DoubleValue Duration { get; set; }
    public DoubleValue StartTime { get; set; }
    public DoubleValue EndTime {
      get {
        return new DoubleValue(Duration.Value + StartTime.Value);
      }
    }
    public IntValue JobNr { get; set; }


    public ScheduledTask(int resNr, double startTime, double duration, int jobNr)
      : base() {
      Duration = new DoubleValue (duration);
      ResourceNr = new IntValue (resNr);
      StartTime = new DoubleValue (startTime);
      JobNr = new IntValue (jobNr);
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[" + JobNr + "," + ResourceNr + "]");
      return sb.ToString();
    }
  }
}
