﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Encodings.SchedulingEncoding.Interfaces;
using HeuristicLab.Optimization;
using HeuristicLab.Operators;

namespace HeuristicLab.Encodings.SchedulingEncoding.Encodings {
  [Item("Scheduling Manipulator", "A JSSP manipulation operation.")]
  [StorableClass]
  public abstract class SchedulingManipulator<T> : SingleSuccessorOperator, ISchedulingManipulator, IStochasticOperator where T : Item, ISchedulingEncoding {
    #region ISchedulingManipulator Members

    public ILookupParameter<T> SchedulingSolutionParameter {
      get { return (ILookupParameter<T>)Parameters["SchedulingSolution"]; }
    }

    #endregion

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    [StorableConstructor]
    protected SchedulingManipulator(bool deserializing) : base(deserializing) { }
    protected SchedulingManipulator(SchedulingManipulator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public SchedulingManipulator()
      : base() {
        Parameters.Add(new LookupParameter<T>("SchedulingSolution", "The JSSP solution to be manipulated."));
        Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
    }

  }
}
