﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Encodings.SchedulingEncoding;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.Scheduling.Decoders {
  public static class GTAlgorithmUtils {

    public static ItemList<Task> GetEarliestNotScheduledTasks(ItemList<Job> jobData) {
      ItemList<Task> result = new ItemList<Task>();
      foreach (Job j in jobData) {
        foreach (Task t in j.Tasks) {
          if (!t.IsScheduled.Value) {
            result.Add(t);
            break;
          }
        }
      }
      return result;
    }
    public static Task GetTaskWithMinimalEC(ItemList<Task> earliestTasksList, Schedule schedule) {
      double minEct = double.MaxValue;
      Task result = null;
      foreach (Task t in earliestTasksList) {
        double ect = ComputeEarliestCompletionTime(t, schedule);
        if (ect < minEct) {
          result = t;
          minEct = ect;
        }
      }
      return result;
    }
    public static ItemList<Task> GetConflictSetForTask(Task conflictedTask, ItemList<Task> earliestTasksList, ItemList<Job> jobData, Schedule schedule) {
      ItemList<Task> result = new ItemList<Task>();
      double conflictedCompletionTime = ComputeEarliestCompletionTime(conflictedTask, schedule);
      result.Add(conflictedTask);
      foreach (Task t in earliestTasksList) {
        if (t.ResourceNr.Value == conflictedTask.ResourceNr.Value) {
          if (ComputeEarliestStartTime(t, schedule) < conflictedCompletionTime)
            result.Add(t);
        }
      }
      return result;
    }
    
    public static double ComputeEarliestStartTime(Task t, Schedule schedule) {
      ScheduledTask previousTask = schedule.GetLastScheduledTaskForJobNr (t.JobNr.Value);
      Resource affectedResource = schedule.Resources[t.ResourceNr.Value];
      double lastMachineEndTime = affectedResource.TotalDuration.Value;
      double previousJobTaskEndTime = 0;
      if (previousTask != null)
        previousJobTaskEndTime = previousTask.EndTime.Value;

      return Math.Max(previousJobTaskEndTime, lastMachineEndTime);
    }
    public static double ComputeEarliestCompletionTime(Task t, Schedule schedule) {
      return ComputeEarliestStartTime(t, schedule) + t.Duration.Value;
    }
  }
}
