﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Encodings.SchedulingEncoding;

namespace HeuristicLab.Problems.Scheduling.Evaluators {
  [Item("Makespan Evaluator", "Represents an evaluator using the maximum makespan of a schedule.")]
  [StorableClass]
  public class MakespanEvaluator : SchedulingEvaluator {
    [StorableConstructor]
    protected MakespanEvaluator(bool deserializing) : base(deserializing) { }
    protected MakespanEvaluator(MakespanEvaluator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MakespanEvaluator(this, cloner);
    }

    public MakespanEvaluator () : base () { }

    protected override DoubleValue evaluate(Schedule schedule) {
      DoubleValue quality = new DoubleValue (0);
      foreach (Resource r in schedule.Resources) {
        if (r.TotalDuration.Value > quality.Value) {
          quality.Value = r.TotalDuration.Value;
        }
      }
      return quality;
    }

    public override IOperation Apply() {
      return base.Apply();
    }
  }
}
